@@ -794,6 +794,9 @@ class Store(object):
                     if inner_deletions:
                         deletions.extend(inner_deletions)
 
+                elif key == '..':
+                    self.outer.apply_update(value, state)
+
             if delete_keys is not None:
                 # delete a list of paths
                 here = self.path_for()
@@ -1284,7 +1287,7 @@ class Experiment(object):
         emitter_config['experiment_id'] = self.experiment_id
         self.emitter = get_emitter(emitter_config)
 
-        self.local_time = 0.0
+        self.experiment_time = 0.0
 
         # run the derivers
         self.send_updates([])
@@ -1354,13 +1357,7 @@ class Experiment(object):
             # if not parallel, perform a normal invocation
             return self.invoke(process, interval, states)
 
-    def process_update(self, path, process, interval):
-        state = self.state.get_path(path)
-        process_topology = get_in(self.topology, path)
-
-        # translate the values from the tree structure into the form
-        # that this process expects, based on its declared topology
-        states = state.outer.schema_topology(process.schema, process_topology)
+    def process_update(self, path, process, store, states, interval):
 
         update = self.invoke_process(
             process,
@@ -1368,9 +1365,22 @@ class Experiment(object):
             interval,
             states)
 
-        absolute = Defer(update, invert_topology, (path, process_topology))
+        absolute = Defer(update, invert_topology, (path, store.topology))
 
-        return absolute, state
+        return absolute, store
+
+    def process_state(self, path, process):
+        store = self.state.get_path(path)
+
+        # translate the values from the tree structure into the form
+        # that this process expects, based on its declared topology
+        states = store.outer.schema_topology(process.schema, store.topology)
+
+        return store, states
+
+    def calculate_update(self, path, process, interval):
+        store, states = self.process_state(path, process)
+        return self.process_update(path, process, store, states, interval)
 
     def apply_update(self, update, state):
         topology_updates, process_updates, deletions = self.state.apply_update(
@@ -1412,14 +1422,14 @@ class Experiment(object):
                 #  generate_paths() add a schema attribute to the Deriver.
                 #  PyCharm's type check reports:
                 #    Type Process doesn't have expected attribute 'schema'
-                update, state = self.process_update(
+                update, state = self.calculate_update(
                     path, deriver, 0)
                 self.apply_update(update.get(), state)
 
     def emit_data(self):
         data = self.state.emit_data()
         data.update({
-            'time': self.local_time})
+            'time': self.experiment_time})
         emit_config = {
             'table': 'history',
             'data': serialize_dictionary(data)}
@@ -1469,8 +1479,14 @@ class Experiment(object):
                 process_time = front[path]['time']
 
                 if process_time <= time:
-                    future = min(process_time + process.local_timestep(), interval)
-                    timestep = future - process_time
+
+                    # get the time step
+                    store, states = self.process_state(path, process)
+                    requested_timestep = process.calculate_timestep(states)
+
+                    # progress only to the end of interval
+                    future = min(process_time + requested_timestep, interval)
+                    process_timestep = future - process_time
 
                     # calculate the update for this process
                     # TODO(jerry): Do something cleaner than having
@@ -1479,14 +1495,24 @@ class Experiment(object):
                     #    Type Process doesn't have expected attribute 'schema'
                     # TODO(chris): Is there any reason to generate a process's
                     #  schema dynamically like this?
-                    update = self.process_update(path, process, timestep)
+                    update = self.process_update(
+                        path, process, store, states, process_timestep)
 
                     # store the update to apply at its projected time
-                    if timestep < full_step:
-                        full_step = timestep
                     front[path]['time'] = future
                     front[path]['update'] = update
 
+                    # absolute timestep
+                    timestep = future - time
+                    if timestep < full_step:
+                        full_step = timestep
+
+                else:
+                    # don't shoot past processes that didn't get to run this time
+                    process_delay = process_time - time
+                    if process_delay < full_step:
+                        full_step = process_delay
+
             if full_step == math.inf:
                 # no processes ran, jump to next process
                 next_event = interval
@@ -1495,13 +1521,15 @@ class Experiment(object):
                         next_event = front[path]['time']
                 time = next_event
             else:
-                # at least one process ran, apply updates and continue
-                future = time + full_step
+                # at least one process ran
+                # increase the time, apply updates, and continue
+                time += full_step
+                self.experiment_time += full_step
 
                 updates = []
                 paths = []
                 for path, advance in front.items():
-                    if advance['time'] <= future:
+                    if advance['time'] <= time:
                         new_update = advance['update']
                         # new_update['_path'] = path
                         updates.append(new_update)
@@ -1510,9 +1538,7 @@ class Experiment(object):
 
                 self.send_updates(updates)
 
-                time = future
-                self.local_time += full_step
-
+                # display and emit
                 if self.progress_bar:
                     print_progress_bar(time, interval)
                 if self.emit_step is None:
@@ -2037,22 +2063,22 @@ def test_complex_topology():
         def ports_schema(self):
             return {
                 'A': {
-                    'a': {'_default': 0},
-                    'b': {'_default': 0},
-                    'c': {'_default': 0}},
+                    'a1': {'_default': 0},
+                    'a2': {'_default': 0},
+                    'a3': {'_default': 0}},
                 'B': {
-                    'd': {'_default': 0},
-                    'e': {'_default': 0}}}
+                    'b1': {'_default': 0},
+                    'b2': {'_default': 0}}}
 
         def next_update(self, timestep, states):
             return {
                 'A': {
-                    'a': states['A']['b'],
-                    'b': states['A']['c'],
-                    'c': states['B']['d'] + states['B']['e']},
+                    'a1': 1,
+                    'a2': 1,
+                    'a3': 1},
                 'B': {
-                    'd': states['A']['a'],
-                    'e': states['B']['e']}}
+                    'b1': -1,
+                    'b2': -1}}
 
     class Qo(Process):
         name = 'qo'
@@ -2060,77 +2086,78 @@ def test_complex_topology():
         def ports_schema(self):
             return {
                 'D': {
-                    'x': {'_default': 0},
-                    'y': {'_default': 0},
-                    'z': {'_default': 0}},
+                    'd1': {'_default': 0},
+                    'd2': {'_default': 0},
+                    'd3': {'_default': 0}},
                 'E': {
-                    'u': {'_default': 0},
-                    'v': {'_default': 0}}}
+                    'e1': {'_default': 0},
+                    'e2': {'_default': 0}}}
 
         def next_update(self, timestep, states):
             return {
                 'D': {
-                    'x': -1,
-                    'y': 12,
-                    'z': states['D']['x'] + states['D']['y']},
+                    'd1': 10,
+                    'd2': 10,
+                    'd3': 10},
                 'E': {
-                    'u': 3,
-                    'v': states['E']['u']}}
+                    'e1': -10,
+                    'e2': -10}}
 
     class PoQo(Composite):
         def generate_processes(self, config=None):
-            p = Po(config)
-            q = Qo(config)
-
             return {
-                'po': p,
-                'qo': q}
+                'po': Po(config),
+                'qo': Qo(config),
+            }
 
         def generate_topology(self, config=None):
             return {
                 'po': {
                     'A': {
                         '_path': ('aaa',),
-                        'b': ('o',)},
-                    'B': ('bbb',)},
+                        'a2': ('x',),
+                        'a3': ('..', 'ccc', 'a3')},
+                    'B': ('bbb',),
+                },
                 'qo': {
                     'D': {
-                        'x': ('aaa', 'a'),
-                        'y': ('aaa', 'o'),
-                        'z': ('ddd', 'z')},
+                        '_path': (),
+                        'd1': ('aaa', 'd1'),
+                        'd2': ('aaa', 'd2'),
+                        'd3': ('ccc', 'd3')},
                     'E': {
-                        'u': ('aaa', 'u'),
-                        'v': ('bbb', 'e')}}}
+                        '_path': (),
+                        'e1': ('aaa', 'x'),
+                        'e2': ('bbb', 'e2')}
+                },
+            }
 
-    initial_state = {
-        'aaa': {
-            'a': 2,
-            'c': 5,
-            'o': 3,
-            'u': 11},
-        'bbb': {
-            'd': 14,
-            'e': 88},
-        'ddd': {
-            'z': 333}}
 
+    # make the experiment
+    outer_path = ('universe', 'agent')
     pq = PoQo({})
-    pq_config = pq.generate()
-    pq_config['initial_state'] = initial_state
+    pq_composite = pq.generate(path=outer_path)
+    experiment = Experiment(pq_composite)
 
-    experiment = Experiment(pq_config)
+    # get the initial state
+    initial_state = experiment.state.get_value()
+    print('time 0:')
+    pp(initial_state)
 
-    pp(experiment.state.get_value())
+    # simulate for 1 second
     experiment.update(1)
 
-    state = experiment.state.get_value()
-    assert state['aaa']['a'] == initial_state['aaa']['a'] + initial_state['aaa']['o'] - 1
-    assert state['aaa']['o'] == initial_state['aaa']['o'] + initial_state['aaa']['c'] + 12
-    assert state['aaa']['c'] == initial_state['aaa']['c'] + initial_state['bbb']['d'] + initial_state['bbb']['e']
-    assert state['aaa']['u'] == initial_state['aaa']['u'] + 3
-    assert state['bbb']['d'] == initial_state['bbb']['d'] + initial_state['aaa']['a']
-    assert state['bbb']['e'] == initial_state['bbb']['e'] + initial_state['bbb']['e'] + initial_state['aaa']['u']
-    assert state['ddd']['z'] == initial_state['ddd']['z'] + initial_state['aaa']['a'] + initial_state['aaa']['o']
+    next_state = experiment.state.get_value()
+    print('time 1:')
+    pp(next_state)
+
+    # pull out the agent state
+    initial_agent_state = initial_state['universe']['agent']
+    agent_state = next_state['universe']['agent']
+
+    assert agent_state['aaa']['a1'] == initial_agent_state['aaa']['a1'] + 1
+    assert agent_state['aaa']['x'] == initial_agent_state['aaa']['x'] - 9
+    assert agent_state['ccc']['a3'] == initial_agent_state['ccc']['a3'] + 1
 
 
 def test_multi():
@@ -2263,8 +2290,7 @@ if __name__ == '__main__':
     # test_multi()
     # test_sine()
     # test_parallel()
-    # test_complex_topology()
+    test_complex_topology()
     # test_multi_port_merge()
     # test_2_store_1_port()
-
-    test_units()
+    # test_units()
